/* spviavoice.c
 *
 * Copyright 2001, 2002 Sun Microsystems, Inc.,
 * Copyright 2001, 2002 BAUM Retec, A.G.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <unistd.h>

#include "spviavoice.h"


/*#define VV_DEBUG*/

#define VERSION_LENGTH 20 

#define ERR_LENGTH    100
#define ERR_NO  	7

#define INITERROR       0
#define SYSTEMERROR     1
#define MEMORYERROR     2
#define MODULELOADERROR 3
#define DELTAERROR      4
#define SYNTHERROR      5 
#define DEVICEERROR     6

#define VV_ERR          0
#define VV_OK           1

/* __ GLOBALS ___*/

ECIHand hECI;
char eci_err_msg[ERR_LENGTH];
char *eci_errors[ERR_NO] =
{
    "Could not create ECI handle. Check installation.\n \
    Possible causes:\n \
    \tIBM ViaVoice TTS RTK has not been installed on this machine.\n \
    \tno exported ECIINI variable in /etc/profile.\n \
    \teci.ini file not in path referenced by ECIINI, nor in current directory.\n \
    \teci.ini corrupted.\n \
    \tlibrary /usr/lib/libibmeci50.so corrupted or missing.\n\
    \tlanguage files referenced in eci.ini file corrupted or missing.\n \
    Re-install IBM ViaVoice TTS RTK.\n",

    "Operating system returned an error.",
    "System resources low.",
    "Unable to load necessary program module.",
    "Error in Delta program.",
    "Error in synthesis engine.",
    "Error using sound device."
}; 

typedef struct voice {
    int gender    ;/* 0|1   - male | female              [`vgN] */
    int headsize  ;/* 0-100 - 100 = very deep voice      [`vhN] */ 
    int pitch_base;/* 0-100(40-422 RWU)                  [`vbN] */
    int pitch_fluc;/* 0-100 - 100 =very exicited         [`vfN] */
    int roughness ;/* 0-100 - smooth - rough             [`vrN] */
    int speed     ;/* 0-250(70-1297 RWU)                        */
		   /*  number of words spoken per minute [`vsN] */
    int volume    ;/* 0-100(1-65535 RWU)                 [`vvN] */
    int breathy   ;/* 0-100 -  100=whisper               [`vyN] */
}vv_voice;

typedef struct param {
    int SynthMode         ;/* 0|1     - Sentence (default)| Manual 
			      (the way input buffer is handled) 		   */
    int InputType         ;/* 0|1     - Plain (default)| Annoteted 		   */
    int TextMode          ;/* 0|1|2|3 - Default | AlphaSpell | AllSpell | IRCSpell */
    int Dictionary        ;/* 0|1     - Abbreviation dictionaries used | unused    */
    int SampleRate        ;/* 0|1|2   - 8000 | 11,025 (default) | 22,050 
			                     samples per second                    */
    int WantPhonemeIndices;/* 0|1     - Phoneme indeces are not (default) | 
			      are genrated (used with callbacks) 		   */
    int RealWorldUnits    ;/* 0|1     - ECI units(default) |RWU			   */
    int LanguageDialect   ;/* eciGeneralAmericanEnglish | eciBritishEnglish        | 
			      eciCastilianSpanish       | eciMexicanSpanich        | 
			      eciStandardFrench         | eciStandardGerman        | 
			      eciStandardItalian        | eciBrazilianPortuguese   */
    int NumberMode        ;/* 0|1     - Pronounce 4 digit numbers as "nonyears" | 
						"years"(default)                   */
    int PhrasePrediction  ;
    int NumParams         ;
}vv_param;


/*  __ API ___ */

void vv_error(char* source)
{
    int err_flags;

    eciErrorMessage(hECI,eci_err_msg);
    fprintf(stderr,"%s : %s\n", source, eci_err_msg);

    err_flags = eciProgStatus(hECI);
    if ( err_flags & ECI_SYSTEMERROR )
	fprintf(stderr,"%s : %s\n", source, eci_errors[SYSTEMERROR]);
    if ( err_flags & ECI_MEMORYERROR )
	fprintf(stderr,"%s : %s\n", source, eci_errors[MEMORYERROR]);    
    if ( err_flags & ECI_MODULELOADERROR )
	fprintf(stderr,"%s : %s\n", source, eci_errors[MODULELOADERROR]);
    if ( err_flags & ECI_DELTAERROR )
	fprintf(stderr,"%s : %s\n", source, eci_errors[DELTAERROR]);
    if ( err_flags & ECI_SYNTHERROR )
	fprintf(stderr,"%s : %s\n", source, eci_errors[SYNTHERROR]);
    if ( err_flags & ECI_DEVICEERROR )
	fprintf(stderr,"%s : %s\n", source, eci_errors[DEVICEERROR]);

    eciClearErrors(hECI);
}


void vv_say (const char *voice, const char *string)
{

#ifdef VV_DEBUG	
	fprintf (stderr, "vv_say: voice:\"%s\" text:\"%s\"\n", voice, string);
#endif	
	if ( !eciAddText(hECI,string) )
	{
	    vv_error("vv_say [eciAddText]");
	}
	
	if ( !eciSynthesize(hECI) )
	{
	    vv_error("vv_say [eciSynthesize]");
	}
/*	
	if ( eciSynchronize(hECI) == false )
	{
	    vv_error("vv_say");
	}
*/	
	if ( !eciClearInput(hECI) )
	{
	    vv_error("vv_say [eciClearInput]");
	}

}

void vv_speak (SRS_VOICE* voice, SRS_TEXT* text )
{
	/* !!! TBI !!! map all voice parameter to the acutal voice here */
	if (text)
	{
		if (voice)
		{
			vv_say (voice->TTSVoiceName, text->Text);
		}
		else
		{
			vv_say (voice->TTSVoiceName, text->Text);
		}
	}
}

void vv_shut_up()
{
  if ( eciStop(hECI) == false)
  {
    vv_error("vv_shut_up");
  }
}
/*
void vv_pause(Boolean pause_on)
{
  if ( eciPause(hECI,pause_on) == false)
  {
    vv_error("vv_pause");
  }

}
*/
int vv_init (TTS_ENGINE* engine)
{	
	char version[VERSION_LENGTH];
	hECI = eciNew();	
	
	if ( hECI == NULL_ECI_HAND )
	{
		fprintf(stderr,"vv_init : %s\n", eci_errors[INITERROR]);
		return -1;		
	}
	
	eciVersion(version);
#ifdef VV_DEBUG
	fprintf(stderr, "Welcome to Via Voice TTS Version %s.\n",version);			
#endif	
	/* eciTestPhrase(hECI);this could be used to test the ViaVoice */
	
	/* parse annotated text */
	eciSetParam(hECI,eciInputType,1);
	
	/* synthesis and input clearing is controlled by commands only */
	eciSetParam(hECI,eciSynthMode,1);

   	/* !!! TBI !!! store the callback address */
   				
   	/* fill the engine structure with the functions */ 
   	engine->ShutUp    = vv_shut_up;
	engine->Speak     = vv_speak;
	engine->Terminate = vv_terminate;
   			   			
	return VV_OK;

}

void vv_terminate ()
{
    if ( hECI == NULL_ECI_HAND ) return;
    
    eciDelete (hECI);
    hECI = NULL_ECI_HAND;
}

